/*
 *	Tykkipeli 2
 *	Weapon/shield selection during game
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <unistd.h>

#include "defines.h"
#include "tp2.h"
#include "game_screen.h"
#include "players.h"
#include "weapons.h"
#include "shields.h"
#include "info_windows.h"
#include "control_panel.h"
#include "shield_defines.h"
#include "control_panel_defines.h"

#include "hr_colors.h"
#include "hr_mouse.h"
#include "hr_bobs.h"
#include "hr_buttons.h"
#include "hr_zones.h"
#include "hr_fonts.h"
#include "hr_defines.h"
#include "hr_general.h"
#include "hr_input_handler.h"
#include "hr_keyboard.h"
#include "hr_screens.h"
#include "hr_windows.h"
#include "hr_graphics.h"

/* Delay before scrolls.*/
#define SCROLL_DELAY              10000
/* Some zone/button indexes.*/
#define SCROLL_UP                 66101
#define SCROLL_DOWN               66102
/* Selection window size and position.*/
#define ITEMWINDOW_X 50
#define ITEMWINDOW_Y 30
#define ITEMWINDOW_WIDTH (WIDTH-(ITEMWINDOW_X*2))
#define ITEMWINDOW_HEIGHT (HEIGHT-(ITEMWINDOW_Y*2))
/* Number of visible items in list.*/
#define MAX_ITEMS_ON_SCREEN       ((ITEMWINDOW_HEIGHT-19)/10)
/* Set this to 0 to make info window as big as item window.
 * Increase size to make info window smaller.*/
#define INFO_WINDOW_OFFSET 10


struct hr_zone *old_zone=NULL;

/* This func returns 1 if hr_input_handler() should exit
 * (this happend when active selection_zone[] changes).*/
int snoop_selection_zone()
{
	struct hr_zone *zp;
	/* Get current zone.*/
	zp=hr_get_zone(hr.firstzone,hr_mouse.x,hr_mouse.y,ZONE_ENABLED);
	/* Return 1 if mouse has been moved from one zone to another. */
	if(zp!=old_zone)
	  return 1;
	else
	  return 0;
}


/* Active weapon/Shield selection */
void change_active_item(struct player *p,int m)
{
	struct hr_zone *zp,*ozp,*selection_zone[MAX_ITEMS_ON_SCREEN],
	  *name_zone,*amount_zone,*zp1,*zp2;
	struct hr_button *b,*changebutton,*sliderbutton,*infobutton;
	struct weapon_list *wl,**ownedw=NULL;
	struct shield_list *sl,**owneds=NULL;
	struct hr_window *itemwindow;
	unsigned char tmp[BUFFER_SIZE];
	int owned_weapons,owned_shields,l,lx,yl,oyl,c,
	  selection_made,x1,y1,x2,y2,xd2,mode_change,visible,info_mode=0;
	void *ptr=NULL;
	
	/* Create array of pointers to owned weapons.*/
	owned_weapons=l=0;
	wl=p->weapons;
	while(wl!=NULL){
		if((wl->amount)!=0){
			++owned_weapons;
			ownedw=realloc(ownedw,(sizeof(struct weapon_list *)*owned_weapons));
			*(ownedw+l++)=wl;
		}
		wl=wl->next;
	}
	/* Create array of pointers to owned shields.*/
	owned_shields=l=0;
	sl=p->shields;
	while(sl!=NULL){
		/* Note: The (None) shield is included in this list even tho
		 * none are really "owned" */
		if((sl->type->index==CHARS2INT("None"))||((sl->amount)>0)){
			++owned_shields;
			owneds=realloc(owneds,(sizeof(struct shield_list *)*owned_shields));
			*(owneds+l++)=sl;
		}
		sl=sl->next;
	}
	
	/* Ready to roll */
	itemwindow=hr_window_open(ITEMWINDOW_X,ITEMWINDOW_Y,
									  ITEMWINDOW_WIDTH,ITEMWINDOW_HEIGHT,
									  hr.def_colors,NONE);
	itemwindow->gadgets=hr_create_window_gadgets(1);
	hr.default_button=(itemwindow->gadgets->button)[0]=
	  hr_create_button(0,0,0,0,
							 hr.font8x8,hr.def_colors,
							 "Cancel",NULL,'c',
							 BUTTON_ENABLED|BUTTON_INVISIBLE);
	hr_window_refresh(itemwindow);
	
	
	x1=itemwindow->zone->x1;
	y1=itemwindow->zone->y1;
	x2=itemwindow->zone->x2;
	y2=itemwindow->zone->y2;
	xd2=(x1+x2)/2;
	changebutton=hr_create_button(x1+2,y1+2,xd2,y1+13,
											hr.font8x8,hr.def_colors,
											NULL,NULL,' ',
											BUTTON_ENABLED);
	hr_write(xd2+6,y1+4,1,0,
				"owned by",hr.font8x8,STRING_START,STRING_END,NONE);
	hr_set_color_index(temp_colors,COLOR_FOREGROUND,p->darkcolor);
	hr_textbox(xd2+76,y1+2,x2-46,y1+13,
				  hr.font8x8,temp_colors,p->name,
				  STRING_START,STRING_END,
				  BUTTON_PRESSED|BUTTON_ENABLED);
	infobutton=hr_create_button(x2-44,y1+2,x2-2,y1+13,
										 hr.font8x8,hr.def_colors,
										 "Info",NULL,'i',
										 BUTTON_ENABLED|BUTTON_SWITCH);
	
	/* Two disabled zones to contain some coordinates.*/
	/* xd2=(y coord of cancel button)-2 */
	xd2=(((itemwindow->gadgets->button)[0])->zone->y1)-2;
	name_zone=hr_create_zone(x1+2,y1+15,x2-108,xd2,NONE);
	amount_zone=hr_create_zone(x2-90,y1+15,x2-2,xd2,NONE);
	hr_bevelbox(name_zone->x1,name_zone->y1,
					name_zone->x2,name_zone->y2,hr.def_colors,BUTTON_PRESSED);
	hr_bevelbox(amount_zone->x1,amount_zone->y1,
					amount_zone->x2,amount_zone->y2,hr.def_colors,BUTTON_PRESSED);
	
	/* Selection zones */
	for(l=0;l<MAX_ITEMS_ON_SCREEN;l++){
		selection_zone[l]=
		  hr_create_zone((name_zone->x1)+2,(name_zone->y1)+2+l*10,
							  (name_zone->x2)-2,
							  (name_zone->y1)+12+l*10,ZONE_ENABLED);
	}
	/* Slider bar */
	sliderbutton=hr_create_button((name_zone->x2)+2,(name_zone->y1),
											(amount_zone->x1)-2,(name_zone->y2),
											hr.font8x8,hr.def_colors,
											NULL,NULL,0,
											BUTTON_VSLIDER|BUTTON_ENABLED|BUTTON_PRESSED
											|BUTTON_NO_TAB_CYCLE);
	hr_clearkey();
	hr_input_handler(NULL,NULL,AMB_RELEASED);

	yl=0;oyl=-666;
	zp=ozp=NULL;
	mode_change=1;
	selection_made=0;
	/* This makes sure list is drawn */
	while(!(selection_made)) {
		while(!(selection_made)) {			
			/* Redraw list. First init buttons and stuff. */
			if((oyl!=yl)||(mode_change)) {
				if(mode_change){
					mode_change=0;
					/* Slider */
					free(sliderbutton->extras);
					l=((m==0)?(owned_weapons):(owned_shields))-MAX_ITEMS_ON_SCREEN;
					visible=5;					/* FIX! Calc correct value. */
					sliderbutton->extras=
					  hr_create_slider(BUTTON_VSLIDER|SLIDER_GET_YVALINT|
											 SLIDER_SET_YVALINT,
											 0,0,0,0,NULL,
											 l,visible,
											 yl,0,&yl);
					/* Buttonzen */ 
					hr_update_button(changebutton,
										  UPDATE_TEXT,(m==0)?("Weapons"):("Shields"));
				}
				/* Update slider. If slider was used, it automagically updates
				 * yl.*/
				hr_update_button(sliderbutton,UPDATE_GFXONLY,NULL);
				/* Clear the name/amoung fields.*/
				/* UGLY! FIX! Only clear area that is INSIDE bevelbox.*/
				hr_clear_zone(name_zone,0,NONE);
				hr_clear_zone(amount_zone,0,NONE);
				hr_bevelbox(name_zone->x1,name_zone->y1,
								name_zone->x2,name_zone->y2,hr.def_colors,
								BUTTON_PRESSED);
				hr_bevelbox(amount_zone->x1,amount_zone->y1,
								amount_zone->x2,amount_zone->y2,hr.def_colors,
								BUTTON_PRESSED);
				/* Go through owned weapons/shields.*/
				for(l=yl,lx=0;
					 l<((m==0)?(owned_weapons):(owned_shields));
					 l++,lx++) {
					/* Break out of the loop if screen is already full */
					if(lx>=MAX_ITEMS_ON_SCREEN)
					  break;
					/* Only print weap/shield info if it fits on screen.*/
					/* Using these makes the source a bit more readable*/
					if(m==0)
					  wl=*(ownedw+l);
					else
					  sl=*(owneds+l);
					/* Only print weapon info if the current weapon is owned.*/
					if(((m==0)&&((wl->amount)!=0))||
						((m==1)&&(((sl->amount)>0)||
									 (sl->type->index==CHARS2INT("None"))))) {
						/* Default color is BLACK */
						c=hr_get_color_index(hr.def_colors,COLOR_DARK);
						/* Print current active weap/shield in WHITE*/
						if(((m==0)&&(p->active_weapon)==(wl->type))||
							((m==1)&&(p->active_shield)==(sl->type)))
						  c=hr_get_color_index(hr.def_colors,COLOR_FOREGROUND);
						/* Print weap/shield name */
						strcpy(tmp,(m==0)?(wl->type->name):(sl->type->name));
						tmp[29]=0;
						hr_write((name_zone->x1)+2,(name_zone->y1)+3+lx*10,
									c,0,tmp,hr.font8x8,STRING_START,STRING_END,NONE);
						/* Print weap/shield amount */
						if(m==0){
							if((wl->amount)<0)
							  strcpy(tmp,"Inf.");
							else
							  sprintf(tmp,"%d",wl->amount);
						}
						else{
							if((yl+l)==0)
							  /* (None) */
							  sprintf(tmp,"-");
							else
							  sprintf(tmp,"%d%%",
										 ((sl->amount)*100)/(sl->type->efficiency));
						}
						hr_write((amount_zone->x2)-2-strlen(tmp)*8,
									(amount_zone->y1)+3+lx*10,
									c,0,tmp,hr.font8x8,STRING_START,STRING_END,NONE);
					}
				}
				oyl=yl;
				usleep(SCROLL_DELAY);
			}
			
			/* Store current zone number to old_zone */
			old_zone=zp;
			/* Get button & zone etc*/
			b=hr_input_handler(NULL,&snoop_selection_zone,
									 KBD_PRESSED|AMB_PRESSED);
			/* Check if the player wants to abort */
			if(HR_RMB)
			  selection_made=1;
			else {
				zp=hr_get_zone(hr.firstzone,hr_mouse.x,hr_mouse.y,ZONE_ENABLED);
				if((HR_LMB) && (zp!=NULL)){
					for(l=0;l<MAX_ITEMS_ON_SCREEN;l++){
						if((yl+l)<((m==0)?(owned_weapons):(owned_shields))){
							if(zp==selection_zone[l]){
								if(m==0) {
									/* Weapons */
									wl=*(ownedw+yl+l);
									if(info_mode)
									  ptr=(void *)(wl->type);
									else
									  p->active_weapon=(wl->type);
								}
								else {
									/* Shields */
									sl=*(owneds+yl+l);
									if(info_mode)
									  ptr=(void *)(sl->type);
									else
									  p->active_shield=(sl->type);
								}
								if(info_mode)
								  item_information(m,ptr,
														 x1+INFO_WINDOW_OFFSET,
														 y1+INFO_WINDOW_OFFSET,
														 x2-INFO_WINDOW_OFFSET,
														 y2-INFO_WINDOW_OFFSET);
								
								else
								  selection_made=1;
								oyl=-666;
							}
						}
					}
				}
				/*Buttons*/
				b=hr_input_handler(b,NULL,AMB_RELEASED);
				zp=hr_get_zone(hr.firstzone,hr_mouse.x,hr_mouse.y,ZONE_ENABLED);
				/* Mode changed? */
				if(b==changebutton){
					mode_change=1;
					m=((m==0)?(1):(0));
					yl=0;
					/* :-) Make sure the list is redrawn */
					oyl=-666;
				}
				/* "Info" pressed? */
				else if(b==infobutton){
					info_mode^=1;
				}
				/* "Cancel" pressed? */
				else if(b==(itemwindow->gadgets->button)[0]){
					selection_made=1;
				}
				/* Item box */
				zp1=zp2=NULL;
				if(ozp!=zp){
					for(l=0;l<MAX_ITEMS_ON_SCREEN;l++){
						if((yl+l)<((m==0)?(owned_weapons):(owned_shields))){
							/* Remove old borders.*/
							if(ozp==selection_zone[l])
							  zp1=ozp;
							/* Draw new borders*/
							else if(zp==selection_zone[l])
							  zp2=zp;
						}
					}
					if(zp1!=NULL)
					  hr_box(zp1->x1,zp1->y1,zp1->x2,zp1->y2,
								hr_get_color_index(hr.def_colors,COLOR_BACKGROUND),
								NONE);
					if(zp2!=NULL)
					  hr_bevelbox(zp2->x1,zp2->y1,zp2->x2,zp2->y2,
									  hr.def_colors,BUTTON_PRESSED);
					ozp=zp;
				}
				/* Keyboard controls */
				switch(hr_keyboard.lastkey){
				 case 27:
					if(hr_ask_confirmation("Abort game?")){
						game_flags |= GAME_ABORTED;
						selection_made=1;
					}
					break;
				}			
				
			}
		}
		/* Don't check blahblah if the game was aborted.*/
		if(!(game_flags & GAME_ABORTED)){
			/* Do not allow aborting when the player has ran out of active
			 * weapons! */
			wl=p->weapons;
			while((wl->type)!=(p->active_weapon))
			  wl=wl->next;
			if((wl->amount)==0)
			  selection_made=0;
		}
	}
	hr_input_handler(NULL,NULL,AMB_RELEASED);
	free(ownedw);
	free(owneds);
	/* Delete zones and buttons inside the window.*/
	hr_delete_zone(name_zone);
	hr_delete_zone(amount_zone);
	for(l=0;l<MAX_ITEMS_ON_SCREEN;l++)
	  hr_delete_zone(selection_zone[l]);
	/* Slider bar */
	hr_delete_button(sliderbutton);
	hr_delete_button(changebutton);
	hr_delete_button(infobutton);
	/* Delete all buttons and zones. They are re-created in
	 * create_control_panel().*/
	hr_window_close(itemwindow);
}


/* Cycles through weapons/shields with non-zero default_amount.
 * If all default_amounts are 0, this will loop forever.*/
void change_default_item(int rosterindex,int m)
{
	struct weapon *w;
	struct shield *s;
	switch(m){
	 case 0:
		/* Change to next weapon */
		w=get_weapon(roster[rosterindex].default_weapon);
		do{
			w=w->next;
			if(w==NULL)
			  w=firstweapon;
		}while(w->default_amount==0);
		roster[rosterindex].default_weapon=(w->index);
		break;
	 case 1:
		/* Change to next shield */
		s=get_shield(roster[rosterindex].default_shield);
		do{
			s=s->next;
			if(s==NULL)
			  s=firstshield;
		}while(s->default_amount==0);
		roster[rosterindex].default_shield=(s->index);
		break;
	}
}
